@@ -1,18 +1,19 @@
 """Cloudnet product quality checks."""
-import dataclasses
 import datetime
 import json
 import logging
 import os
 import re
-from dataclasses import dataclass
 from enum import Enum
+from os import PathLike
 from pathlib import Path
+from typing import Iterable, NamedTuple
 
 import netCDF4
 import numpy as np
 import scipy.stats
 from numpy import ma
+from requests import RequestException
 
 from . import utils
 from .variables import LEVELS, VARIABLES, Product
@@ -27,91 +28,82 @@ CF_STANDARD_NAMES_XML = os.path.join(DATA_PATH, "cf-standard-name-table.xml")
 CF_REGION_NAMES_XML = os.path.join(DATA_PATH, "standardized-region-list.xml")
 
 
-class ErrorLevel(str, Enum):
+class ErrorLevel(Enum):
     WARNING = "warning"
     ERROR = "error"
     INFO = "info"
 
 
-@dataclass
-class TestReport:
-    testId: str
-    exceptions: list[dict]
+class TestException(NamedTuple):
+    result: ErrorLevel
+    message: str
 
-    def values(self):
+
+class TestReport(NamedTuple):
+    test_id: str
+    exceptions: list[TestException]
+
+
+class FileReport(NamedTuple):
+    timestamp: datetime.datetime
+    qc_version: str
+    tests: list[TestReport]
+
+    def to_dict(self) -> dict:
         return {
-            field.name: getattr(self, field.name)
-            for field in dataclasses.fields(self)
-            if getattr(self, field.name) is not None
+            "timestamp": self.timestamp.isoformat(),
+            "qcVersion": self.qc_version,
+            "tests": [
+                {
+                    "testId": test.test_id,
+                    "exceptions": [
+                        {"result": exception.result.value, "message": exception.message}
+                        for exception in test.exceptions
+                    ],
+                }
+                for test in self.tests
+            ],
         }
 
 
-@dataclass
-class FileReport:
-    timestamp: str
-    qcVersion: str
-    tests: list[dict]
-
-
 def run_tests(
-    filename: Path | str,
-    cloudnet_file_type: str | None = None,
+    filename: str | PathLike,
+    product: Product | str | None = None,
     ignore_tests: list[str] | None = None,
-) -> dict:
-    if isinstance(filename, str):
-        filename = Path(filename)
+) -> FileReport:
+    filename = Path(filename)
+    if isinstance(product, str):
+        product = Product(product)
     with netCDF4.Dataset(filename) as nc:
-        if cloudnet_file_type is None:
+        if product is None:
             try:
-                cloudnet_file_type = nc.cloudnet_file_type
-            except AttributeError:
-                logging.error(
-                    "No cloudnet_file_type global attribute found, can not run tests. "
+                product = Product(nc.cloudnet_file_type)
+            except AttributeError as exc:
+                raise ValueError(
+                    "No 'cloudnet_file_type' global attribute found, can not run tests. "
                     "Is this a legacy file?"
-                )
-                return {}
+                ) from exc
         logging.debug(f"Filename: {filename.stem}")
-        logging.debug(f"File type: {cloudnet_file_type}")
-        test_reports: list[dict] = []
+        logging.debug(f"File type: {product}")
+        test_reports: list[TestReport] = []
         for cls in Test.__subclasses__():
             if ignore_tests and cls.__name__ in ignore_tests:
                 continue
-            test_instance = cls(nc, filename, cloudnet_file_type)
-            if cloudnet_file_type in test_instance.products:
+            test_instance = cls(nc, filename, product)
+            if product in test_instance.products:
                 test_instance.run()
-                for exception in test_instance.report.values()["exceptions"]:
-                    assert exception["result"] in (
+                for exception in test_instance.report.exceptions:
+                    assert exception.result in (
                         ErrorLevel.ERROR,
                         ErrorLevel.WARNING,
                         ErrorLevel.INFO,
                     )
-                test_reports.append(test_instance.report.values())
+                test_reports.append(test_instance.report)
     return FileReport(
-        timestamp=f"{datetime.datetime.now().isoformat()}Z",
-        qcVersion=__version__,
+        timestamp=datetime.datetime.now(tz=datetime.timezone.utc),
+        qc_version=__version__,
         tests=test_reports,
-    ).__dict__
-
-
-def test(
-    name: str,
-    description: str,
-    products: list[Product] | None = None,
-    ignore_products: list[Product] | None = None,
-):
-    """Decorator for the tests."""
-
-    def fun(cls):
-        setattr(cls, "name", name)
-        setattr(cls, "description", description)
-        if products is not None:
-            setattr(cls, "products", [member.value for member in products])
-        if ignore_products is not None:
-            prods = list(set(getattr(cls, "products")) - set(ignore_products))
-            setattr(cls, "products", prods)
-        return cls
-
-    return fun
+    )
 
 
 class Test:
@@ -119,14 +111,14 @@ class Test:
 
     name: str
     description: str
-    products: list[str] = Product.all()
+    products: Iterable[Product] = Product.all()
 
-    def __init__(self, nc: netCDF4.Dataset, filename: Path, cloudnet_file_type: str):
+    def __init__(self, nc: netCDF4.Dataset, filename: Path, product: Product):
         self.filename = filename
         self.nc = nc
-        self.cloudnet_file_type = cloudnet_file_type
+        self.product = product
         self.report = TestReport(
-            testId=self.__class__.__name__,
+            test_id=self.__class__.__name__,
             exceptions=[],
         )
 
@@ -135,10 +127,7 @@ class Test:
 
     def _add_message(self, message: str | list, severity: ErrorLevel):
         self.report.exceptions.append(
-            {
-                "message": utils.format_msg(message),
-                "result": severity,
-            }
+            TestException(result=severity, message=utils.format_msg(message))
         )
 
     def _add_info(self, message: str | list):
@@ -151,7 +140,7 @@ class Test:
         self._add_message(message, ErrorLevel.ERROR)
 
     def _read_config_keys(self, config_section: str) -> np.ndarray:
-        field = "all" if "attr" in config_section else self.cloudnet_file_type
+        field = "all" if "attr" in config_section else self.product.value
         keys = METADATA_CONFIG[config_section][field].split(",")
         return np.char.strip(keys)
 
@@ -159,7 +148,7 @@ class Test:
         return {
             name: var
             for name, var in VARIABLES.items()
-            if var.required is not None and self.cloudnet_file_type in var.required
+            if var.required is not None and self.product in var.required
         }
 
     def _get_required_variable_names(self) -> set:
@@ -185,17 +174,32 @@ class Test:
         date_in_file = [int(getattr(self.nc, x)) for x in ("year", "month", "day")]
         return datetime.date(*date_in_file)
 
+    def _get_duration(self) -> datetime.timedelta:
+        now = datetime.datetime.now(tz=datetime.timezone.utc)
+        if now.date() == self._get_date():
+            midnight = now.replace(hour=0, minute=0, second=0, microsecond=0)
+            duration = now - midnight
+        else:
+            duration = datetime.timedelta(days=1)
+        return duration
+
 
 # --------------------#
 # ------ Infos ------ #
 # --------------------#
 
 
-@test("Variable outliers", "Find suspicious data values.")
 class FindVariableOutliers(Test):
+    name = "Variable outliers"
+    description = "Find suspicious data values."
+
     def run(self):
         for key, limits_str in DATA_CONFIG.items("limits"):
-            if key == "zenith_angle" and self.cloudnet_file_type.startswith("mwr-"):
+            if key == "zenith_angle" and self.product in (
+                Product.MWR_L1C,
+                Product.MWR_SINGLE,
+                Product.MWR_MULTI,
+            ):
                 continue
             limits = [float(x) for x in limits_str.split(",")]
             if key in self.nc.variables:
@@ -212,12 +216,11 @@ class FindVariableOutliers(Test):
                     self._add_info(msg)
 
 
-@test(
-    "Radar folding",
-    "Test for radar folding.",
-    products=[Product.RADAR, Product.CATEGORIZE],
-)
 class FindFolding(Test):
+    name = "Radar folding"
+    description = "Test for radar folding."
+    products = [Product.RADAR, Product.CATEGORIZE]
+
     def run(self):
         key = "v"
         v_threshold = 8
@@ -234,37 +237,34 @@ class FindFolding(Test):
             )
 
 
-@test(
-    "Data coverage",
-    "Test that file contains enough data.",
-)
 class TestDataCoverage(Test):
+    name = "Data coverage"
+    description = "Test that file contains enough data."
+
     RESOLUTIONS = {
-        Product.MODEL: datetime.timedelta(hours=1),
-        Product.MWR_MULTI: datetime.timedelta(minutes=20),
-        Product.WEATHER_STATION: datetime.timedelta(minutes=1),
         Product.DISDROMETER: datetime.timedelta(minutes=1),
+        Product.L3_CF: datetime.timedelta(hours=1),
+        Product.L3_IWC: datetime.timedelta(hours=1),
+        Product.L3_LWC: datetime.timedelta(hours=1),
+        Product.MODEL: datetime.timedelta(hours=1),
+        Product.MWR: datetime.timedelta(minutes=2),
+        Product.MWR_MULTI: datetime.timedelta(minutes=20),
+        Product.MWR_SINGLE: datetime.timedelta(minutes=2),
+        Product.WEATHER_STATION: datetime.timedelta(minutes=1),
     }
     DEFAULT_RESOLUTION = datetime.timedelta(seconds=30)
 
     def run(self):
-        time = self.nc["time"][:]
+        time = np.array(self.nc["time"][:])
         time_unit = datetime.timedelta(hours=1)
         try:
             n_time = len(time)
         except (TypeError, ValueError):
             return
-        if n_time == 0:
+        if n_time < 2:
             return
-        expected_res = self.RESOLUTIONS.get(
-            self.cloudnet_file_type, self.DEFAULT_RESOLUTION
-        )
-        now = datetime.datetime.now(tz=datetime.timezone.utc)
-        if now.date() == self._get_date():
-            midnight = now.replace(hour=0, minute=0, second=0, microsecond=0)
-            duration = now - midnight
-        else:
-            duration = datetime.timedelta(days=1)
+        expected_res = self.RESOLUTIONS.get(self.product, self.DEFAULT_RESOLUTION)
+        duration = self._get_duration()
         bins = max(1, duration // expected_res)
         hist, _bin_edges = np.histogram(
             time, bins=bins, range=(0, duration / time_unit)
@@ -285,12 +285,16 @@ class TestDataCoverage(Test):
             )
 
 
-@test(
-    "Variable names",
-    "Check that variables have expected names.",
-    ignore_products=[Product.MODEL],
-)
 class TestVariableNamesDefined(Test):
+    name = "Variable names"
+    description = "Check that variables have expected names."
+    products = Product.all() - {
+        Product.MODEL,
+        Product.L3_CF,
+        Product.L3_IWC,
+        Product.L3_LWC,
+    }
+
     def run(self):
         for key in self.nc.variables.keys():
             if key not in VARIABLES:
@@ -302,34 +306,46 @@ class TestVariableNamesDefined(Test):
 # ---------------------- #
 
 
-@test("Units", "Check that variables have expected units.")
 class TestUnits(Test):
+    name = "Units"
+    description = "Check that variables have expected units."
+
     def run(self):
         self._test_variable_attribute("units")
 
 
-@test(
-    "Long names",
-    "Check that variables have expected long names.",
-    ignore_products=[Product.MODEL],
-)
 class TestLongNames(Test):
+    name = "Long names"
+    description = "Check that variables have expected long names."
+    products = Product.all() - {
+        Product.MODEL,
+        Product.L3_CF,
+        Product.L3_IWC,
+        Product.L3_LWC,
+    }
+
     def run(self):
         self._test_variable_attribute("long_name")
 
 
-@test(
-    "Standard names",
-    "Check that variable have expected standard names.",
-    ignore_products=[Product.MODEL],
-)
 class TestStandardNames(Test):
+    name = "Standard names"
+    description = "Check that variable have expected standard names."
+    products = Product.all() - {
+        Product.MODEL,
+        Product.L3_CF,
+        Product.L3_IWC,
+        Product.L3_LWC,
+    }
+
     def run(self):
         self._test_variable_attribute("standard_name")
 
 
-@test("Data types", "Check that variables have expected data types.")
 class TestDataTypes(Test):
+    name = "Data types"
+    description = "Check that variables have expected data types."
+
     def run(self):
         for key in self.nc.variables:
             if key not in VARIABLES:
@@ -345,18 +361,16 @@ class TestDataTypes(Test):
                 self._add_warning(msg)
 
 
-# @test(
-#     "Time data type",
-#     "Check that time vector is in double precision.",
+# class TestTimeVectorDataType(Test):
+#     name = "Time data type"
+#     description = "Check that time vector is in double precision."
 #     products=[
 #         Product.RADAR,
 #         Product.LIDAR,
 #         Product.MWR,
 #         Product.DISDROMETER,
 #         Product.WEATHER_STATION,
-#     ],
-# )
-# class TestTimeVectorDataType(Test):
+#     ]
 #     def run(self):
 #         key = "time"
 #         received = self.nc.variables[key].dtype.name
@@ -366,8 +380,10 @@ class TestDataTypes(Test):
 #             self._add_warning(msg)
 
 
-@test("Global attributes", "Check that file contains required global attributes.")
 class TestGlobalAttributes(Test):
+    name = "Global attributes"
+    description = "Check that file contains required global attributes."
+
     REQUIRED_ATTRS = {
         "year",
         "month",
@@ -400,7 +416,7 @@ class TestGlobalAttributes(Test):
 
     def _optional_attr(self, name: str, product: Product) -> bool:
         return (
-            name == "references"
+            name in ("references", "pid")
             or name.endswith("_version")
             or (
                 product == Product.MODEL
@@ -411,36 +427,48 @@ class TestGlobalAttributes(Test):
 
     def run(self):
         nc_keys = set(self.nc.ncattrs())
-        required_attrs = self._required_attrs(self.cloudnet_file_type)
+        required_attrs = self._required_attrs(self.product)
         missing_keys = required_attrs - nc_keys
         for key in missing_keys:
             self._add_warning(f"Attribute '{key}' is missing.")
         extra_keys = nc_keys - required_attrs
         for key in extra_keys:
-            if not self._optional_attr(key, self.cloudnet_file_type):
+            if not self._optional_attr(key, self.product):
                 self._add_warning(f"Unknown attribute '{key}' found.")
 
 
-@test(
-    "Median LWP",
-    "Test that median of LWP values is within a reasonable range.",
-    [Product.MWR, Product.CATEGORIZE],
-)
 class TestMedianLwp(Test):
+    name = "Median LWP"
+    description = "Test that LWP data are valid."
+    products = [Product.MWR, Product.CATEGORIZE]
+
     def run(self):
         key = "lwp"
         if key not in self.nc.variables:
             self._add_error(f"'{key}' is missing.")
             return
+        data = self.nc.variables[key][:]
+        mask_percentage = ma.count_masked(data) / data.size * 100
+        if mask_percentage > 20:
+            msg = (
+                f"{round(mask_percentage,1)} % of '{key}' data points are masked "
+                f"due to low quality data."
+            )
+            if mask_percentage > 60:
+                self._add_warning(msg)
+            else:
+                self._add_info(msg)
         limits = [-0.5, 10]
-        median_lwp = ma.median(self.nc.variables[key][:]) / 1000  # g -> kg
+        median_lwp = ma.median(data) / 1000  # g -> kg
         if median_lwp < limits[0] or median_lwp > limits[1]:
             msg = utils.create_out_of_bounds_msg(key, *limits, median_lwp)
             self._add_warning(msg)
 
 
-@test("Attribute outliers", "Find suspicious values in global attributes.")
 class FindAttributeOutliers(Test):
+    name = "Attribute outliers"
+    description = "Find suspicious values in global attributes."
+
     def run(self):
         try:
             year = int(self.nc.year)
@@ -453,42 +481,67 @@ class FindAttributeOutliers(Test):
             self._add_warning("Invalid date attributes.")
 
 
-@test(
-    "LDR values",
-    "Test that LDR values are proper.",
-    products=[Product.RADAR, Product.CATEGORIZE],
-)
 class TestLDR(Test):
+    name = "LDR values"
+    description = "Test that LDR values are proper."
+    products = [Product.RADAR, Product.CATEGORIZE]
+
     def run(self):
-        if "ldr" in self.nc.variables:
-            ldr = self.nc["ldr"][:]
-            if ldr.mask.all():
-                self._add_warning("LDR exists but all the values are invalid.")
+        has_ldr = "ldr" in self.nc.variables or "sldr" in self.nc.variables
+        has_v = "v" in self.nc.variables
+        if has_v and has_ldr:
+            v = self.nc["v"][:]
+            ldr = (
+                self.nc["ldr"][:] if "ldr" in self.nc.variables else self.nc["sldr"][:]
+            )
+            v_count = ma.count(v)
+            ldr_count = ma.count(ldr)
+            if v_count > 0 and (ldr_count / v_count * 100) < 0.1:
+                self._add_warning("LDR exists in less than 0.1 % of pixels.")
+
+
+class TestMask(Test):
+    name = "Data mask"
+    description = "Test that data are not completely masked."
+    products = [Product.RADAR]
+
+    def run(self):
+        if not np.any(~self.nc["v"][:].mask):
+            self._add_error("All data are masked.")
 
 
-@test(
-    "Range correction", "Test that beta is range corrected.", products=[Product.LIDAR]
-)
 class TestIfRangeCorrected(Test):
+    name = "Range correction"
+    description = "Test that beta is range corrected."
+    products = [Product.LIDAR]
+
     def run(self):
         try:
             range_var = self.nc["range"]
             beta_raw = self.nc["beta_raw"]
         except IndexError:
             return
-        n_top_ranges = 200
+
+        n_top_ranges = len(range_var) // 2
         x = range_var[-n_top_ranges:] ** 2
         y = np.std(beta_raw[:, -n_top_ranges:], axis=0)
-        res = scipy.stats.pearsonr(x, y)
+        sgl_res = scipy.stats.siegelslopes(y, x)
+        residuals = np.abs(y - (sgl_res.intercept + sgl_res.slope * x))
+        outliers = residuals > 20 * np.percentile(
+            residuals, 25
+        )  # Ad hoc outlier detection
+        res = scipy.stats.pearsonr(x[~outliers], y[~outliers])
         if res.statistic < 0.75:
             self._add_warning("Data might not be range corrected.")
 
 
-@test(
-    "Floating-point values",
-    "Test for special floating-point values which may indicate problems with the processing.",
-)
 class TestFloatingPointValues(Test):
+    name = "Floating-point values"
+    description = (
+        "Test for special floating-point values "
+        "which may indicate problems with the processing."
+    )
+
     def run(self):
         for name, variable in self.nc.variables.items():
             if variable.dtype.kind != "f":
@@ -504,12 +557,11 @@ class TestFloatingPointValues(Test):
 # -------------------- #
 
 
-@test(
-    "Beta presence",
-    "Test that one beta variable exists.",
-    products=[Product.LIDAR],
-)
 class TestLidarBeta(Test):
+    name = "Beta presence"
+    description = "Test that one beta variable exists."
+    products = [Product.LIDAR]
+
     def run(self):
         valid_keys = {"beta", "beta_1064", "beta_532", "beta_355"}
         for key in valid_keys:
@@ -518,8 +570,10 @@ class TestLidarBeta(Test):
         self._add_error("No valid beta variable found.")
 
 
-@test("Time vector", "Test that time vector is continuous.")
 class TestTimeVector(Test):
+    name = "Time vector"
+    description = "Test that time vector is continuous."
+
     def run(self):
         time = self.nc["time"][:]
         try:
@@ -544,8 +598,10 @@ class TestTimeVector(Test):
             self._add_error(msg)
 
 
-@test("Variables", "Check that file contains required variables.")
 class TestVariableNames(Test):
+    name = "Variables"
+    description = "Check that file contains required variables."
+
     def run(self):
         keys_in_file = set(self.nc.variables.keys())
         required_keys = self._get_required_variable_names()
@@ -554,13 +610,51 @@ class TestVariableNames(Test):
             self._add_error(f"'{key}' is missing.")
 
 
+class TestModelData(Test):
+    name = "Model data"
+    description = "Test that model data are valid."
+    products = [Product.MODEL]
+
+    def run(self):
+        time = np.array(self.nc["time"][:])
+        time_unit = datetime.timedelta(hours=1)
+
+        try:
+            n_time = len(time)
+        except (TypeError, ValueError):
+            return
+        if n_time < 2:
+            return
+
+        duration = self._get_duration()
+        should_be_data_until = duration / time_unit
+
+        for key in ("temperature", "pressure", "q"):
+            if key not in self.nc.variables:
+                continue
+            data = self.nc[key][:]
+            missing_hours = [
+                int(hour)
+                for ind, hour in enumerate(time)
+                if hour < should_be_data_until
+                and ma.count_masked(data[ind, :]) == data.shape[1]
+            ]
+            if not missing_hours:
+                continue
+            noun, verb = ("Hour", "is") if len(missing_hours) == 1 else ("Hours", "are")
+            values = utils.format_list(utils.integer_ranges(missing_hours), "and")
+            self._add_error(f"{noun} {values} {verb} missing from variable '{key}'.")
+
+
 # ------------------------------#
 # ------ Error / Warning ------ #
 # ----------------------------- #
 
 
-@test("CF conventions", "Test compliance with the CF metadata conventions.")
 class TestCFConvention(Test):
+    name = "CF conventions"
+    description = "Test compliance with the CF metadata conventions."
+
     def run(self):
         from cfchecker import cfchecks  # pylint: disable=import-outside-toplevel
 
@@ -588,27 +682,29 @@ class TestCFConvention(Test):
                 self._add_message(msg, severity)
 
 
-@test(
-    "Instrument PID",
-    "Test that valid instrument PID exists.",
-    [
+class TestInstrumentPid(Test):
+    name = "Instrument PID"
+    description = "Test that valid instrument PID exists."
+    products = [
         Product.MWR,
         Product.LIDAR,
         Product.RADAR,
         Product.DISDROMETER,
         Product.DOPPLER_LIDAR,
         Product.WEATHER_STATION,
-    ],
-)
-class TestInstrumentPid(Test):
+    ]
+
     data: dict = {}
 
     def run(self):
         if self._check_exists():
-            self.data = utils.fetch_pid(self.nc.instrument_pid)
-            self._check_serial()
-            self._check_model_name()
-            self._check_model_identifier()
+            try:
+                self.data = utils.fetch_pid(self.nc.instrument_pid)
+                self._check_serial()
+                self._check_model_name()
+                self._check_model_identifier()
+            except RequestException:
+                self._add_info("Failed to fetch instrument PID")
 
     def _check_exists(self) -> bool:
         key = "instrument_pid"
@@ -704,19 +800,20 @@ class TestInstrumentPid(Test):
     }
 
     SOURCE_TO_IDENTIFIER = {
-        "BASTA": ["https://vocabulary.actris.nilu.no/actris_vocab/BASTA"],
+        "BASTA": ["https://vocabulary.actris.nilu.no/actris_vocab/MeteomodemBASTA"],
         "METEK MIRA-35": [
-            "https://vocabulary.actris.nilu.no/actris_vocab/METEKMIRA35",
-            "https://vocabulary.actris.nilu.no/actris_vocab/METEKMIRA35S",
+            "https://vocabulary.actris.nilu.no/actris_vocab/MetekMIRA35",
+            "https://vocabulary.actris.nilu.no/actris_vocab/MetekMIRA35S",
+            "https://vocabulary.actris.nilu.no/actris_vocab/MetekMIRA35C",
         ],
         "OTT HydroMet Parsivel2": [
             "https://vocabulary.actris.nilu.no/actris_vocab/OTTParsivel2"
         ],
         "RAL Space Copernicus": [
-            "https://vocabulary.actris.nilu.no/actris_vocab/UFAMCopernicus"
+            "https://vocabulary.actris.nilu.no/actris_vocab/RALCopernicus"
         ],
         "RAL Space Galileo": [
-            "https://vocabulary.actris.nilu.no/actris_vocab/UFAMGalileo"
+            "https://vocabulary.actris.nilu.no/actris_vocab/RALGalileo"
         ],
         "RPG-Radiometer Physics HATPRO": [
             "https://vocabulary.actris.nilu.no/actris_vocab/RPGHATPRO"
