@@ -1,13 +1,15 @@
 """Variable definitions"""
 # pylint: disable=too-many-lines
+from __future__ import annotations
+
 from collections.abc import Callable
 from enum import Enum
-from typing import Literal, NamedTuple
+from typing import Iterable, Literal, NamedTuple
 
 import netCDF4
 
 
-class Product(str, Enum):
+class Product(Enum):
     # Level 1b
     RADAR = "radar"
     LIDAR = "lidar"
@@ -29,10 +31,14 @@ class Product(str, Enum):
     DRIZZLE = "drizzle"
     MWR_SINGLE = "mwr-single"
     MWR_MULTI = "mwr-multi"
+    # Level 3
+    L3_CF = "l3-cf"
+    L3_IWC = "l3-iwc"
+    L3_LWC = "l3-lwc"
 
     @classmethod
-    def all(cls) -> list[str]:
-        return [e.value for e in cls]
+    def all(cls) -> set[Product]:
+        return set(cls)
 
 
 Level = Literal["1b", "1c", "2", "3"]
@@ -56,6 +62,9 @@ LEVELS: dict[Product, Level] = {
     Product.DRIZZLE: "2",
     Product.MWR_SINGLE: "2",
     Product.MWR_MULTI: "2",
+    Product.L3_CF: "3",
+    Product.L3_IWC: "3",
+    Product.L3_LWC: "3",
 }
 
 
@@ -71,7 +80,7 @@ class Variable(NamedTuple):
     units: str | Callable[[netCDF4.Dataset], str] | None = "1"
     dtype: str = Dtype.FLOAT
     standard_name: str | None = None
-    required: list[str] | None = None
+    required: Iterable[Product] | None = None
 
 
 def time_units(nc: netCDF4.Dataset) -> str:
@@ -83,21 +92,30 @@ VARIABLES = {
     # Required in RADAR Level 1b file
     # -------------------------------
     "radar_frequency": Variable(
-        long_name="Radar transmit frequency", units="GHz", required=[Product.RADAR]
+        long_name="Radar transmit frequency",
+        units="GHz",
+        required=[Product.RADAR],
     ),
     "Zh": Variable(
         long_name="Radar reflectivity factor",
         units="dBZ",
-        required=[Product.RADAR, Product.RAIN_RADAR],
+        required=[
+            Product.RADAR,
+            Product.RAIN_RADAR,
+        ],
     ),
     "nyquist_velocity": Variable(
-        long_name="Nyquist velocity", units="m s-1", required=[Product.RADAR]
+        long_name="Nyquist velocity",
+        units="m s-1",
+        required=[Product.RADAR],
     ),
     # -------------------------------
     # Required in LIDAR Level 1b file
     # -------------------------------
     "wavelength": Variable(
-        long_name="Laser wavelength", units="nm", required=[Product.LIDAR]
+        long_name="Laser wavelength",
+        units="nm",
+        required=[Product.LIDAR],
     ),
     "zenith_angle": Variable(
         long_name="Zenith angle",
@@ -159,20 +177,32 @@ VARIABLES = {
     # Required in CATEGORIZE Level 1c file
     # ------------------------------------
     "lidar_wavelength": Variable(
-        long_name="Laser wavelength", units="nm", required=[Product.CATEGORIZE]
+        long_name="Laser wavelength",
+        units="nm",
+        required=[Product.CATEGORIZE],
     ),
     "insect_prob": Variable(
-        long_name="Insect probability", required=[Product.CATEGORIZE]
+        long_name="Insect probability",
+        required=[Product.CATEGORIZE],
     ),
     "uwind": Variable(
-        long_name="Zonal wind", units="m s-1", required=[Product.CATEGORIZE]
+        long_name="Zonal wind",
+        units="m s-1",
+        required=[Product.CATEGORIZE],
     ),
     "vwind": Variable(
-        long_name="Meridional wind", units="m s-1", required=[Product.CATEGORIZE]
+        long_name="Meridional wind",
+        units="m s-1",
+        required=[Product.CATEGORIZE],
+    ),
+    "q": Variable(
+        long_name="Specific humidity",
+        required=[Product.CATEGORIZE],
     ),
-    "q": Variable(long_name="Specific humidity", required=[Product.CATEGORIZE]),
     "Tw": Variable(
-        long_name="Wet-bulb temperature", units="K", required=[Product.CATEGORIZE]
+        long_name="Wet-bulb temperature",
+        units="K",
+        required=[Product.CATEGORIZE],
     ),
     "category_bits": Variable(
         long_name="Target categorization bits",
@@ -190,13 +220,18 @@ VARIABLES = {
         required=[Product.CATEGORIZE],
     ),
     "quality_bits": Variable(
-        long_name="Data quality bits", dtype=Dtype.INT, required=[Product.CATEGORIZE]
+        long_name="Data quality bits",
+        dtype=Dtype.INT,
+        required=[Product.CATEGORIZE],
     ),
     "beta": Variable(
         long_name="Attenuated backscatter coefficient",
         units="sr-1 m-1",
         # Lidar has its own beta test
-        required=[Product.CATEGORIZE, Product.DOPPLER_LIDAR],
+        required=[
+            Product.CATEGORIZE,
+            Product.DOPPLER_LIDAR,
+        ],
     ),
     "beta_error": Variable(
         long_name="Error in attenuated backscatter coefficient",
@@ -234,7 +269,9 @@ VARIABLES = {
         required=[Product.CATEGORIZE],
     ),
     "model_time": Variable(
-        long_name="Model time UTC", units=time_units, required=[Product.CATEGORIZE]
+        long_name="Model time UTC",
+        units=time_units,
+        required=[Product.CATEGORIZE],
     ),
     "model_height": Variable(
         long_name="Height of model variables above mean sea level",
@@ -280,7 +317,10 @@ VARIABLES = {
     "lwc": Variable(
         long_name="Liquid water content",
         units="kg m-3",
-        required=[Product.LWC, Product.RAIN_RADAR],
+        required=[
+            Product.LWC,
+            Product.RAIN_RADAR,
+        ],
     ),
     "lwc_error": Variable(
         long_name="Random error in liquid water content, one standard deviation",
@@ -329,7 +369,9 @@ VARIABLES = {
     # Required in DRIZZLE Level 2 file
     # --------------------------------
     "Do": Variable(
-        long_name="Drizzle median diameter", units="m", required=[Product.DRIZZLE]
+        long_name="Drizzle median diameter",
+        units="m",
+        required=[Product.DRIZZLE],
     ),
     "mu": Variable(
         long_name="Drizzle droplet size distribution shape parameter",
@@ -341,7 +383,8 @@ VARIABLES = {
         required=[Product.DRIZZLE],
     ),
     "beta_corr": Variable(
-        long_name="Lidar backscatter correction factor", required=[Product.DRIZZLE]
+        long_name="Lidar backscatter correction factor",
+        required=[Product.DRIZZLE],
     ),
     "drizzle_N": Variable(
         long_name="Drizzle number concentration",
@@ -364,7 +407,9 @@ VARIABLES = {
         required=[Product.DRIZZLE],
     ),
     "v_air": Variable(
-        long_name="Vertical air velocity", units="m s-1", required=[Product.DRIZZLE]
+        long_name="Vertical air velocity",
+        units="m s-1",
+        required=[Product.DRIZZLE],
     ),
     "Do_error": Variable(
         units="dB",
@@ -435,7 +480,9 @@ VARIABLES = {
     # Required in IER Level 2 file
     # ----------------------------
     "ier": Variable(
-        long_name="Ice effective radius", units="m", required=[Product.IER]
+        long_name="Ice effective radius",
+        units="m",
+        required=[Product.IER],
     ),
     "ier_inc_rain": Variable(
         long_name="Ice effective radius including rain",
@@ -456,7 +503,9 @@ VARIABLES = {
     # Required in DER Level 2 file
     # ----------------------------
     "der": Variable(
-        long_name="Droplet effective radius", units="m", required=[Product.DER]
+        long_name="Droplet effective radius",
+        units="m",
+        required=[Product.DER],
     ),
     "der_error": Variable(
         long_name="Absolute error in droplet effective radius",
@@ -474,7 +523,8 @@ VARIABLES = {
         required=[Product.DER],
     ),
     "N_scaled": Variable(
-        long_name="Cloud droplet number concentration", required=[Product.DER]
+        long_name="Cloud droplet number concentration",
+        required=[Product.DER],
     ),
     "der_retrieval_status": Variable(
         long_name="Droplet effective radius retrieval status",
@@ -501,7 +551,9 @@ VARIABLES = {
         required=[Product.MWR_L1C],
     ),
     "sideband_IF_separation": Variable(
-        long_name="Sideband IF separation", units="GHz", required=[Product.MWR_L1C]
+        long_name="Sideband IF separation",
+        units="GHz",
+        required=[Product.MWR_L1C],
     ),
     "freq_shift": Variable(
         long_name="Frequency shift of the microwave channels",
@@ -531,19 +583,26 @@ VARIABLES = {
     "ir_beamwidth": Variable(
         long_name="Beam width of the infrared radiometer",
         units="degree",
-        required=[Product.MWR_L1C],
     ),
     "n_sidebands": Variable(
-        long_name="Number of sidebands", dtype=Dtype.INT, required=[Product.MWR_L1C]
+        long_name="Number of sidebands",
+        dtype=Dtype.INT,
+        required=[Product.MWR_L1C],
     ),
     "pointing_flag": Variable(
-        long_name="Pointing flag", dtype=Dtype.INT, required=[Product.MWR_L1C]
+        long_name="Pointing flag",
+        dtype=Dtype.INT,
+        required=[Product.MWR_L1C],
     ),
     "t_amb": Variable(
-        long_name="Ambient target temperature", units="K", required=[Product.MWR_L1C]
+        long_name="Ambient target temperature",
+        units="K",
+        required=[Product.MWR_L1C],
     ),
     "t_rec": Variable(
-        long_name="Receiver physical temperature", units="K", required=[Product.MWR_L1C]
+        long_name="Receiver physical temperature",
+        units="K",
+        required=[Product.MWR_L1C],
     ),
     "t_sta": Variable(
         long_name="Receiver temperature stability",
@@ -551,29 +610,133 @@ VARIABLES = {
         required=[Product.MWR_L1C],
     ),
     "quality_flag_status": Variable(
-        long_name="Quality flag status", dtype=Dtype.INT, required=[Product.MWR_L1C]
+        long_name="Quality flag status",
+        dtype=Dtype.INT,
+        required=[Product.MWR_L1C],
     ),
     "liquid_cloud_flag": Variable(
-        long_name="Liquid cloud flag", dtype=Dtype.INT, required=[Product.MWR_L1C]
+        long_name="Liquid cloud flag",
+        dtype=Dtype.INT,
+        required=[Product.MWR_L1C],
     ),
     "liquid_cloud_flag_status": Variable(
         long_name="Liquid cloud flag status",
         dtype=Dtype.INT,
         required=[Product.MWR_L1C],
     ),
-    "ir_wavelength": Variable(long_name="Wavelength of infrared channels", units="m"),
-    "ir_bandwidth": Variable(long_name="Bandwidth of infrared channels", units="m"),
-    "irt": Variable(long_name="Infrared brightness temperatures", units="K"),
+    "ir_wavelength": Variable(
+        long_name="Wavelength of infrared channels",
+        units="m",
+    ),
+    "ir_bandwidth": Variable(
+        long_name="Bandwidth of infrared channels",
+        units="m",
+    ),
+    "irt": Variable(
+        long_name="Infrared brightness temperatures",
+        units="K",
+    ),
     "ir_azimuth_angle": Variable(
-        long_name="Infrared sensor azimuth angle", units="degree"
+        long_name="Infrared sensor azimuth angle",
+        units="degree",
     ),
     "ir_zenith_angle": Variable(
-        long_name="Infrared sensor zenith angle", units="degree"
+        long_name="Infrared sensor zenith angle",
+        units="degree",
+    ),
+    # ---------------------------
+    # Required in MWR SINGLE file
+    # ---------------------------
+    "iwv_quality_flag": Variable(
+        long_name="Quality flag",
+        units="1",
+        required=[Product.MWR_SINGLE],
+        dtype=Dtype.INT,
+    ),
+    "iwv_quality_flag_status": Variable(
+        long_name="Quality flag status",
+        units="1",
+        required=[Product.MWR_SINGLE],
+        dtype=Dtype.INT,
+    ),
+    "absolute_humidity_quality_flag": Variable(
+        long_name="Quality flag",
+        units="1",
+        required=[Product.MWR_SINGLE],
+        dtype=Dtype.INT,
+    ),
+    "absolute_humidity_quality_flag_status": Variable(
+        long_name="Quality flag status",
+        units="1",
+        required=[Product.MWR_SINGLE],
+        dtype=Dtype.INT,
+    ),
+    "lwp_quality_flag": Variable(
+        long_name="Quality flag",
+        units="1",
+        required=[Product.MWR_SINGLE],
+        dtype=Dtype.INT,
+    ),
+    "lwp_quality_flag_status": Variable(
+        long_name="Quality flag status",
+        units="1",
+        required=[Product.MWR_SINGLE],
+        dtype=Dtype.INT,
+    ),
+    "lwp_offset": Variable(
+        long_name="Subtracted offset correction of liquid water path",
+        units="kg m-2",
+        required=[Product.MWR_SINGLE],
+    ),
+    "absolute_humidity": Variable(
+        long_name="Absolute humidity",
+        units="kg m-3",
+        required=[Product.MWR_SINGLE],
+    ),
+    # ----------------------------------------
+    # Required in MWR MULTI and MWR MULTI file
+    # ----------------------------------------
+    "potential_temperature": Variable(
+        long_name="Potential temperature",
+        units="K",
+        required=[
+            Product.MWR_SINGLE,
+            Product.MWR_MULTI,
+        ],
     ),
-    "absolute_humidity": Variable(long_name="Absolute humidity", units="kg m-3"),
-    "potential_temperature": Variable(long_name="Potential temperature", units="K"),
     "equivalent_potential_temperature": Variable(
-        long_name="Equivalent potential temperature", units="K"
+        long_name="Equivalent potential temperature",
+        units="K",
+        required=[
+            Product.MWR_SINGLE,
+            Product.MWR_MULTI,
+        ],
+    ),
+    "relative_humidity": Variable(
+        long_name="Relative humidity",
+        standard_name="relative_humidity",
+        required=[
+            Product.MWR_SINGLE,
+            Product.MWR_MULTI,
+        ],
+    ),
+    "temperature_quality_flag": Variable(
+        long_name="Quality flag",
+        units="1",
+        required=[
+            Product.MWR_SINGLE,
+            Product.MWR_MULTI,
+        ],
+        dtype=Dtype.INT,
+    ),
+    "temperature_quality_flag_status": Variable(
+        long_name="Quality flag status",
+        units="1",
+        required=[
+            Product.MWR_SINGLE,
+            Product.MWR_MULTI,
+        ],
+        dtype=Dtype.INT,
     ),
     # -------------------------
     # Required in several files
@@ -583,7 +746,6 @@ VARIABLES = {
         units="m s-1",
         standard_name="rainfall_rate",
         required=[
-            Product.CATEGORIZE,
             Product.DISDROMETER,
             Product.WEATHER_STATION,
             Product.RAIN_RADAR,
@@ -592,7 +754,10 @@ VARIABLES = {
     "range": Variable(
         long_name="Range from instrument",
         units="m",
-        required=[Product.RADAR, Product.LIDAR],
+        required=[
+            Product.RADAR,
+            Product.LIDAR,
+        ],
     ),
     "v": Variable(
         long_name="Doppler velocity",
@@ -605,42 +770,55 @@ VARIABLES = {
         ],
     ),
     "temperature": Variable(
-        long_name="Temperature", units="K", required=[Product.MODEL, Product.CATEGORIZE]
+        long_name="Temperature",
+        units="K",
+        required=[
+            Product.MODEL,
+            Product.CATEGORIZE,
+            Product.MWR_MULTI,
+        ],
     ),
     "pressure": Variable(
-        long_name="Pressure", units="Pa", required=[Product.MODEL, Product.CATEGORIZE]
+        long_name="Pressure",
+        units="Pa",
+        required=[
+            Product.MODEL,
+            Product.CATEGORIZE,
+        ],
     ),
     "lwp": Variable(
         long_name="Liquid water path",
         units="kg m-2",
         standard_name="atmosphere_cloud_liquid_water_content",
-        required=[Product.MWR, Product.CATEGORIZE, Product.LWC, Product.MWR_SINGLE],
+        required=[
+            Product.MWR,
+            Product.CATEGORIZE,
+            Product.LWC,
+            Product.MWR_SINGLE,
+        ],
     ),
     "lwp_error": Variable(
         long_name="Error in liquid water path",
         units="kg m-2",
-        required=[Product.CATEGORIZE, Product.LWC],
-    ),
-    "lwp_offset": Variable(
-        long_name="Subtracted offset correction of liquid water path",
-        units="kg m-2",
-        required=[Product.MWR_SINGLE],
+        required=[
+            Product.CATEGORIZE,
+            Product.LWC,
+        ],
     ),
     "height": Variable(
         long_name="Height above mean sea level",
         units="m",
         standard_name="height_above_mean_sea_level",
-        required=[
-            p
-            for p in Product.all()
-            if p
-            not in (
-                Product.MWR,
-                Product.DISDROMETER,
-                Product.WEATHER_STATION,
-                Product.MWR_L1C,
-            )
-        ],
+        required=Product.all()
+        - {
+            Product.MWR,
+            Product.DISDROMETER,
+            Product.WEATHER_STATION,
+            Product.MWR_L1C,
+            Product.L3_CF,
+            Product.L3_IWC,
+            Product.L3_LWC,
+        },
     ),
     "time": Variable(
         long_name="Time UTC",
@@ -652,7 +830,8 @@ VARIABLES = {
         long_name="Altitude of site",
         units="m",
         standard_name="altitude",
-        required=[p for p in Product.all() if p != Product.MODEL],
+        required=Product.all()
+        - {Product.MODEL, Product.L3_CF, Product.L3_IWC, Product.L3_LWC},
     ),
     "latitude": Variable(
         long_name="Latitude of site",
@@ -669,23 +848,52 @@ VARIABLES = {
     # --------------------------------------------
     # Variables included in some of Level 1b files
     # --------------------------------------------
+    "calibration_offset": Variable(
+        long_name="Radar reflectivity calibration offset",
+        units="dBZ",
+    ),
+    "range_offset": Variable(
+        long_name="Radar range offset",
+        units="m",
+    ),
     "rainfall_rate_1min_total": Variable(
-        long_name="Total precipitation rate", units="m s-1", dtype=Dtype.INT
+        long_name="Total precipitation rate",
+        units="m s-1",
+        dtype=Dtype.INT,
     ),
     "rainfall_rate_1min_solid": Variable(
-        long_name="Solid precipitation rate", units="m s-1", dtype=Dtype.INT
+        long_name="Solid precipitation rate",
+        units="m s-1",
+        dtype=Dtype.INT,
+    ),
+    "snowfall_rate": Variable(
+        long_name="Snowfall rate",
+        units="m s-1",
     ),
-    "snowfall_rate": Variable(long_name="Snowfall rate", units="m s-1"),
     "velocity": Variable(
-        long_name="Center fall velocity of precipitation particles", units="m s-1"
+        long_name="Center fall velocity of precipitation particles",
+        units="m s-1",
+    ),
+    "velocity_spread": Variable(
+        long_name="Width of velocity interval",
+        units="m s-1",
+    ),
+    "velocity_bnds": Variable(
+        long_name="Velocity bounds",
+        units="m s-1",
     ),
-    "velocity_spread": Variable(long_name="Width of velocity interval", units="m s-1"),
-    "velocity_bnds": Variable(long_name="Velocity bounds", units="m s-1"),
     "diameter": Variable(
-        long_name="Center diameter of precipitation particles", units="m"
+        long_name="Center diameter of precipitation particles",
+        units="m",
+    ),
+    "diameter_spread": Variable(
+        long_name="Width of diameter interval",
+        units="m",
+    ),
+    "diameter_bnds": Variable(
+        long_name="Diameter bounds",
+        units="m",
     ),
-    "diameter_spread": Variable(long_name="Width of diameter interval", units="m"),
-    "diameter_bnds": Variable(long_name="Diameter bounds", units="m"),
     "synop_WaWa": Variable(
         long_name="Synop code WaWa",
         dtype=Dtype.INT,
@@ -699,14 +907,26 @@ VARIABLES = {
         long_name="Signal amplitude of the laser strip",
         dtype=Dtype.INT,
     ),
-    "T_sensor": Variable(long_name="Temperature in the sensor housing", units="K"),
-    "I_heating": Variable(long_name="Heating current", units="A"),
+    "T_sensor": Variable(
+        long_name="Temperature in the sensor housing",
+        units="K",
+    ),
+    "I_heating": Variable(
+        long_name="Heating current",
+        units="A",
+    ),
     "kinetic_energy": Variable(
-        long_name="Kinetic energy of the hydrometeors", units="J m-2 h-1"
+        long_name="Kinetic energy of the hydrometeors",
+        units="J m-2 h-1",
+    ),
+    "V_power_supply": Variable(
+        long_name="Power supply voltage",
+        units="V",
     ),
-    "V_power_supply": Variable(long_name="Power supply voltage", units="V"),
     "V_sensor_supply": Variable(
-        long_name="Sensor supply voltage", units="V", dtype=Dtype.INT
+        long_name="Sensor supply voltage",
+        units="V",
+        dtype=Dtype.INT,
     ),
     "state_sensor": Variable(
         long_name="State of the sensor",
@@ -717,10 +937,12 @@ VARIABLES = {
         dtype=Dtype.INT,
     ),
     "number_concentration": Variable(
-        long_name="Number of particles per diameter class", units="m-3 mm-1"
+        long_name="Number of particles per diameter class",
+        units="m-3 mm-1",
     ),
     "fall_velocity": Variable(
-        long_name="Average velocity of each diameter class", units="m s-1"
+        long_name="Average velocity of each diameter class",
+        units="m s-1",
     ),
     "data_raw": Variable(
         long_name="Raw data as a function of particle diameter and velocity",
@@ -741,10 +963,13 @@ VARIABLES = {
         long_name="Skewness of spectra",
     ),
     "azimuth_angle": Variable(
-        long_name="Azimuth angle", units="degree", standard_name="sensor_azimuth_angle"
+        long_name="Azimuth angle",
+        units="degree",
+        standard_name="sensor_azimuth_angle",
     ),
     "beta_raw": Variable(
-        long_name="Attenuated backscatter coefficient", units="sr-1 m-1"
+        long_name="Attenuated backscatter coefficient",
+        units="sr-1 m-1",
     ),
     "iwv": Variable(
         long_name="Integrated water vapour",
@@ -755,9 +980,14 @@ VARIABLES = {
         long_name="Linear depolarisation ratio",
         units="dB",
     ),
-    "sldr": Variable(long_name="Slanted linear depolarisation ratio", units="dB"),
+    "sldr": Variable(
+        long_name="Slanted linear depolarisation ratio",
+        units="dB",
+    ),
     "width": Variable(
-        long_name="Spectral width", units="m s-1", required=[Product.RAIN_RADAR]
+        long_name="Spectral width",
+        units="m s-1",
+        required=[Product.RAIN_RADAR],
     ),
     "calibration_factor": Variable(
         long_name="Attenuated backscatter calibration factor",
@@ -771,39 +1001,87 @@ VARIABLES = {
     "depolarisation_raw": Variable(
         long_name="Lidar volume linear depolarisation ratio",
     ),
-    "file_code": Variable(long_name="File code", dtype=Dtype.INT),
-    "program_number": Variable(long_name="Program number", dtype=Dtype.INT),
-    "model_number": Variable(long_name="Model number", dtype=Dtype.INT),
-    "antenna_separation": Variable(long_name="Antenna separation", units="m"),
-    "antenna_diameter": Variable(long_name="Antenna diameter", units="m"),
-    "antenna_gain": Variable(long_name="Antenna gain", units="dB"),
+    "file_code": Variable(
+        long_name="File code",
+        dtype=Dtype.INT,
+    ),
+    "program_number": Variable(
+        long_name="Program number",
+        dtype=Dtype.INT,
+    ),
+    "model_number": Variable(
+        long_name="Model number",
+        dtype=Dtype.INT,
+    ),
+    "antenna_separation": Variable(
+        long_name="Antenna separation",
+        units="m",
+    ),
+    "antenna_diameter": Variable(
+        long_name="Antenna diameter",
+        units="m",
+    ),
+    "antenna_gain": Variable(
+        long_name="Antenna gain",
+        units="dB",
+    ),
     "half_power_beam_width": Variable(
-        long_name="Half power beam width", units="degree"
+        long_name="Half power beam width",
+        units="degree",
+    ),
+    "dual_polarization": Variable(
+        long_name="Dual polarisation type",
+        dtype=Dtype.INT,
+    ),
+    "sample_duration": Variable(
+        long_name="Sample duration",
+        units="s",
     ),
-    "dual_polarization": Variable(long_name="Dual polarisation type", dtype=Dtype.INT),
-    "sample_duration": Variable(long_name="Sample duration", units="s"),
     "calibration_interval": Variable(
-        long_name="Calibration interval in samples", dtype=Dtype.INT
+        long_name="Calibration interval in samples",
+        dtype=Dtype.INT,
     ),
     "number_of_spectral_samples": Variable(
-        long_name="Number of spectral samples in each chirp sequence", dtype=Dtype.INT
+        long_name="Number of spectral samples in each chirp sequence",
+        dtype=Dtype.INT,
     ),
     "chirp_start_indices": Variable(
-        long_name="Chirp sequences start indices", dtype=Dtype.INT
+        long_name="Chirp sequences start indices",
+        dtype=Dtype.INT,
     ),
     "number_of_averaged_chirps": Variable(
         long_name="Number of averaged chirps in sequence",
         dtype=Dtype.INT,
     ),
-    "integration_time": Variable(long_name="Integration time", units="s"),
-    "range_resolution": Variable(long_name="Vertical resolution of range", units="m"),
-    "FFT_window": Variable(long_name="FFT window type", dtype=Dtype.INT),
+    "integration_time": Variable(
+        long_name="Integration time",
+        units="s",
+    ),
+    "range_resolution": Variable(
+        long_name="Vertical resolution of range",
+        units="m",
+    ),
+    "FFT_window": Variable(
+        long_name="FFT window type",
+        dtype=Dtype.INT,
+    ),
     "input_voltage_range": Variable(
-        long_name="ADC input voltage range (+/-)", units="mV", dtype=Dtype.INT
+        long_name="ADC input voltage range (+/-)",
+        units="mV",
+        dtype=Dtype.INT,
+    ),
+    "noise_threshold": Variable(
+        long_name="Noise filter threshold factor",
+    ),
+    "time_ms": Variable(
+        long_name="Time ms",
+        units="ms",
+        dtype=Dtype.INT,
+    ),
+    "quality_flag": Variable(
+        long_name="Quality flag",
+        dtype=Dtype.INT,
     ),
-    "noise_threshold": Variable(long_name="Noise filter threshold factor"),
-    "time_ms": Variable(long_name="Time ms", units="ms", dtype=Dtype.INT),
-    "quality_flag": Variable(long_name="Quality flag", dtype=Dtype.INT),
     "pc_temperature": Variable(
         long_name="PC temperature",
         units="K",
@@ -823,17 +1101,15 @@ VARIABLES = {
     "status_flag": Variable(
         long_name="Status flag for heater and blower",
     ),
-    "relative_humidity": Variable(
-        long_name="Relative humidity",
-        standard_name="relative_humidity",
-    ),
     "wind_speed": Variable(
         long_name="Wind speed",
         standard_name="wind_speed",
         units="m s-1",
     ),
     "wind_direction": Variable(
-        long_name="Wind direction", units="degree", standard_name="wind_from_direction"
+        long_name="Wind direction",
+        units="degree",
+        standard_name="wind_from_direction",
     ),
     "voltage": Variable(
         long_name="Voltage",
@@ -847,12 +1123,38 @@ VARIABLES = {
         long_name="IF power at ACD",
         units="uW",
     ),
-    "level": Variable(long_name="Model level", units=None, dtype=Dtype.SHORT),
-    "flux_level": Variable(long_name="Model flux level", units=None, dtype=Dtype.SHORT),
-    "sfc_categorical_snow": Variable(long_name="", dtype=Dtype.SHORT),
-    "sfc_categorical_ice": Variable(long_name="", dtype=Dtype.SHORT),
-    "sfc_categorical_freezing_rain": Variable(long_name="", dtype=Dtype.SHORT),
-    "sfc_categorical_rain": Variable(long_name="", dtype=Dtype.SHORT),
+    "level": Variable(
+        long_name="Model level",
+        units=None,
+        dtype=Dtype.SHORT,
+        required=[
+            Product.MODEL,
+            Product.L3_CF,
+            Product.L3_IWC,
+            Product.L3_LWC,
+        ],
+    ),
+    "flux_level": Variable(
+        long_name="Model flux level",
+        units=None,
+        dtype=Dtype.SHORT,
+    ),
+    "sfc_categorical_snow": Variable(
+        long_name="",
+        dtype=Dtype.SHORT,
+    ),
+    "sfc_categorical_ice": Variable(
+        long_name="",
+        dtype=Dtype.SHORT,
+    ),
+    "sfc_categorical_freezing_rain": Variable(
+        long_name="",
+        dtype=Dtype.SHORT,
+    ),
+    "sfc_categorical_rain": Variable(
+        long_name="",
+        dtype=Dtype.SHORT,
+    ),
     "sfc_albedo_sw_direct": Variable(
         long_name="Surface albedo (shortwave direct)",
         # standard_name="surface_albedo_shortwave_direct",
@@ -869,125 +1171,215 @@ VARIABLES = {
         long_name="Surface albedo (longwave diffuse)",
         # standard_name="surface_albedo_longwave_diffuse",
     ),
-    "nfft": Variable(long_name="Number of FFT points", dtype=Dtype.INT),
+    "nfft": Variable(
+        long_name="Number of FFT points",
+        dtype=Dtype.INT,
+    ),
     "nave": Variable(
         long_name="Number of spectral averages (not accounting for overlapping FFTs)",
         dtype=Dtype.INT,
     ),
     "prf": Variable(
-        long_name="Pulse Repetition Frequency", units="Hz", dtype=Dtype.INT
+        long_name="Pulse Repetition Frequency",
+        units="Hz",
+        dtype=Dtype.INT,
+    ),
+    "rg0": Variable(
+        long_name="Number of lowest range gates",
+        dtype=Dtype.INT,
+    ),
+    "SNR": Variable(
+        long_name="Signal-to-noise ratio",
+        units="dB",
     ),
-    "rg0": Variable(long_name="Number of lowest range gates", dtype=Dtype.INT),
-    "SNR": Variable(long_name="Signal-to-noise ratio", units="dB"),
     "radar_pitch": Variable(
-        long_name="Radar pitch angle", units="degree", standard_name="platform_roll"
+        long_name="Radar pitch angle",
+        units="degree",
+        standard_name="platform_roll",
     ),
     "radar_yaw": Variable(
-        long_name="Radar yaw angle", units="degree", standard_name="platform_yaw"
+        long_name="Radar yaw angle",
+        units="degree",
+        standard_name="platform_yaw",
     ),
     "radar_roll": Variable(
-        long_name="Radar roll angle", units="degree", standard_name="platform_roll"
+        long_name="Radar roll angle",
+        units="degree",
+        standard_name="platform_roll",
+    ),
+    "zdr": Variable(
+        long_name="Differential reflectivity",
+        units="dB",
+    ),
+    "rho_hv": Variable(
+        long_name="Correlation coefficient",
+    ),
+    "phi_dp": Variable(
+        long_name="Differential phase",
+        units="rad",
+    ),
+    "srho_hv": Variable(
+        long_name="Slanted correlation coefficient",
+    ),
+    "kdp": Variable(
+        long_name="Specific differential phase shift",
+        units="rad km-1",
     ),
-    "zdr": Variable(long_name="Differential reflectivity", units="dB"),
-    "rho_hv": Variable(long_name="Correlation coefficient"),
-    "phi_dp": Variable(long_name="Differential phase", units="rad"),
-    "srho_hv": Variable(long_name="Slanted correlation coefficient"),
-    "kdp": Variable(long_name="Specific differential phase shift", units="rad km-1"),
     "differential_attenuation": Variable(
-        long_name="Differential attenuation", units="dB km-1"
+        long_name="Differential attenuation",
+        units="dB km-1",
+    ),
+    "synop_WW": Variable(
+        long_name="Synop code WW",
+        dtype=Dtype.INT,
     ),
-    "synop_WW": Variable(long_name="Synop code WW", dtype=Dtype.INT),
     "measurement_quality": Variable(
-        long_name="Measurement quality", units="%", dtype=Dtype.INT
+        long_name="Measurement quality",
+        units="%",
+        dtype=Dtype.INT,
+    ),
+    "status_laser": Variable(
+        long_name="Status of laser",
+        dtype=Dtype.INT,
+    ),
+    "static_signal": Variable(
+        long_name="Static signal",
+        dtype=Dtype.INT,
     ),
-    "status_laser": Variable(long_name="Status of laser", dtype=Dtype.INT),
-    "static_signal": Variable(long_name="Static signal", dtype=Dtype.INT),
     "status_T_laser_analogue": Variable(
-        long_name="Status of laser temperature (analogue)", dtype=Dtype.INT
+        long_name="Status of laser temperature (analogue)",
+        dtype=Dtype.INT,
     ),
     "status_T_laser_digital": Variable(
-        long_name="Status of laser temperature (digital)", dtype=Dtype.INT
+        long_name="Status of laser temperature (digital)",
+        dtype=Dtype.INT,
     ),
     "status_I_laser_analogue": Variable(
-        long_name="Status of laser current (analogue)", dtype=Dtype.INT
+        long_name="Status of laser current (analogue)",
+        dtype=Dtype.INT,
     ),
     "status_I_laser_digital": Variable(
-        long_name="Status of laser current (digital)", dtype=Dtype.INT
+        long_name="Status of laser current (digital)",
+        dtype=Dtype.INT,
     ),
     "status_sensor_supply": Variable(
-        long_name="Status of sensor supply", dtype=Dtype.INT
+        long_name="Status of sensor supply",
+        dtype=Dtype.INT,
     ),
     "status_laser_heating": Variable(
-        long_name="Status of laser heating", dtype=Dtype.INT
+        long_name="Status of laser heating",
+        dtype=Dtype.INT,
     ),
     "status_receiver_heating": Variable(
-        long_name="Status of receiver heating", dtype=Dtype.INT
+        long_name="Status of receiver heating",
+        dtype=Dtype.INT,
     ),
     "status_temperature_sensor": Variable(
-        long_name="Status of temperature sensor", dtype=Dtype.INT
+        long_name="Status of temperature sensor",
+        dtype=Dtype.INT,
     ),
     "status_heating_supply": Variable(
-        long_name="Status of heating supply", dtype=Dtype.INT
+        long_name="Status of heating supply",
+        dtype=Dtype.INT,
     ),
     "status_heating_housing": Variable(
-        long_name="Status of heating housing", dtype=Dtype.INT
+        long_name="Status of heating housing",
+        dtype=Dtype.INT,
     ),
     "status_heating_heads": Variable(
-        long_name="Status of heating heads", dtype=Dtype.INT
+        long_name="Status of heating heads",
+        dtype=Dtype.INT,
     ),
     "status_heating_carriers": Variable(
-        long_name="Status of heating carriers", dtype=Dtype.INT
+        long_name="Status of heating carriers",
+        dtype=Dtype.INT,
+    ),
+    "status_laser_power": Variable(
+        long_name="Status of laser power",
+        dtype=Dtype.INT,
+    ),
+    "T_interior": Variable(
+        long_name="Interior temperature",
+        units="K",
+    ),
+    "T_laser_driver": Variable(
+        long_name="Temperature of laser driver",
+        units="K",
+    ),
+    "T_ambient": Variable(
+        long_name="Ambient temperature",
+        units="K",
     ),
-    "status_laser_power": Variable(long_name="Status of laser power", dtype=Dtype.INT),
-    "T_interior": Variable(long_name="Interior temperature", units="K"),
-    "T_laser_driver": Variable(long_name="Temperature of laser driver", units="K"),
-    "T_ambient": Variable(long_name="Ambient temperature", units="K"),
     "I_mean_laser": Variable(
-        long_name="Mean value of laser current", units="mA", dtype=Dtype.INT
+        long_name="Mean value of laser current",
+        units="mA",
+        dtype=Dtype.INT,
+    ),
+    "V_control": Variable(
+        long_name="Control voltage",
+        units="mV",
+        dtype=Dtype.INT,
     ),
-    "V_control": Variable(long_name="Control voltage", units="mV", dtype=Dtype.INT),
     "V_optical_output": Variable(
-        long_name="Voltage of optical control output", units="mV", dtype=Dtype.INT
+        long_name="Voltage of optical control output",
+        units="mV",
+        dtype=Dtype.INT,
     ),
     "I_heating_laser_head": Variable(
-        long_name="Laser head heating current", units="mA", dtype=Dtype.INT
+        long_name="Laser head heating current",
+        units="mA",
+        dtype=Dtype.INT,
     ),
     "I_heating_receiver_head": Variable(
-        long_name="Receiver head heating current", units="mA", dtype=Dtype.INT
+        long_name="Receiver head heating current",
+        units="mA",
+        dtype=Dtype.INT,
     ),
     "maximum_hail_diameter": Variable(
-        long_name="Maximum hail diameter", units="mm", dtype=Dtype.INT
+        long_name="Maximum hail diameter",
+        units="mm",
+        dtype=Dtype.INT,
     ),
     # rv-polarstern specific
     "beta_1064_raw": Variable(
-        long_name="Attenuated backscatter coefficient", units="sr-1 m-1"
+        long_name="Attenuated backscatter coefficient",
+        units="sr-1 m-1",
     ),
     "beta_1064": Variable(
-        long_name="Attenuated backscatter coefficient", units="sr-1 m-1"
+        long_name="Attenuated backscatter coefficient",
+        units="sr-1 m-1",
     ),
     "beta_532_raw": Variable(
-        long_name="Attenuated backscatter coefficient", units="sr-1 m-1"
+        long_name="Attenuated backscatter coefficient",
+        units="sr-1 m-1",
     ),
     "beta_532": Variable(
-        long_name="Attenuated backscatter coefficient", units="sr-1 m-1"
+        long_name="Attenuated backscatter coefficient",
+        units="sr-1 m-1",
     ),
     "beta_355_raw": Variable(
-        long_name="Attenuated backscatter coefficient", units="sr-1 m-1"
+        long_name="Attenuated backscatter coefficient",
+        units="sr-1 m-1",
     ),
     "beta_355": Variable(
-        long_name="Attenuated backscatter coefficient", units="sr-1 m-1"
+        long_name="Attenuated backscatter coefficient",
+        units="sr-1 m-1",
     ),
     "beta_355_nr": Variable(
-        long_name="Attenuated backscatter coefficient (near range)", units="sr-1 m-1"
+        long_name="Attenuated backscatter coefficient (near range)",
+        units="sr-1 m-1",
     ),
     "beta_355_nr_raw": Variable(
-        long_name="Attenuated backscatter coefficient (near range)", units="sr-1 m-1"
+        long_name="Attenuated backscatter coefficient (near range)",
+        units="sr-1 m-1",
     ),
     "beta_532_nr": Variable(
-        long_name="Attenuated backscatter coefficient (near range)", units="sr-1 m-1"
+        long_name="Attenuated backscatter coefficient (near range)",
+        units="sr-1 m-1",
     ),
     "beta_532_nr_raw": Variable(
-        long_name="Attenuated backscatter coefficient (near range)", units="sr-1 m-1"
+        long_name="Attenuated backscatter coefficient (near range)",
+        units="sr-1 m-1",
     ),
     "calibration_factor_1064": Variable(
         long_name="Attenuated backscatter at 1064 calibration factor",
@@ -1040,6 +1432,13 @@ VARIABLES = {
     "lidar_depolarisation": Variable(
         long_name="Calibrated volume depolarisation (532 nm)"  # in rv-polarstern categorize files
     ),
-    "beamwidthV": Variable(long_name="Vertical angular beamwidth", units="degree"),
-    "beamwidthH": Variable(long_name="Horizontal angular beamwidth", units="degree"),
+    "beamwidthV": Variable(
+        long_name="Vertical angular beamwidth",
+        units="degree",
+    ),
+    "beamwidthH": Variable(
+        long_name="Horizontal angular beamwidth",
+        units="degree",
+    ),
+    "rain_detected": Variable(long_name="Rain detected", dtype=Dtype.INT),
 }
